package lib;

require lib::plugin;

my $VERSION="0.9b11";

sub Version { $VERSION; }

use vars qw (%HTTPUSH_OPTIONS %CURRENT);
use strict;

my $debug_level=0;

sub debug {
  my ($data, $level)=@_;

  if(!$level) {
    $level=1;
  }

  return if ($level > $debug_level);

  open (XLOG,">> httpush.debug");
  flock XLOG, 2;
  print XLOG "($$)# $data\n";
  flock XLOG, 8;
  close XLOG;

  print STDERR "($$)# $data\n";
}

sub announce {
  my($data)=shift;
  print STDERR "=> $data\n";
}

sub get_unique_id {
  my $rand=int(rand()*100000);
  my $id=time().$$.$rand;
  return $id;
}

sub set_debug_level {
  $debug_level=shift;
}

sub my_error {
  my ($error, $data, $code)=@_;

  lib::debug("Displaying error $error");

  if(! $code) {
    $code=502;
  }

  print "HTTP/1.1 $code an error code\r\n";
  print "Content-Type: text/html\r\n";
  print "Connection: Close\r\n";
  print "\r\n";
  print "<HTML>";

  print "<HEAD>";
  print HTMLPrintStyle();

  print "<TITLE>Error retrieving URL</TITLE></HEAD><BODY>";

  if($error==1) {
    print "<H1>Your request was malformed. Please check your proxy settings
           and try again. Remember, this is a HTTP/HTTPS proxy, nothing else.</H1><BR>";
    } elsif($error==2) {
    print "<H1>The address you are trying to access uses an unsupported URI
           scheme. Currently HTTPush only supports HTTP/HTTPS connections</H1><BR>";
    } elsif($error==3) {
    print "<H1>There has been an error connecting to the remote host</H1><I><strong>$data:</strong>The
           address does not exist or it is down.</I><BR>";
    } elsif($error==4) {
    print "<H1>There has been an error while trying to retrieve the learned item. The item doesn't exist anymore.
           </H1><BR>";
    } elsif($error==5) {
    print "<H1>SSL support is unavailable</H1>Please install SSLeay modules if you need HTTPS support.
           <BR>";
    } elsif($error==6) {
    print "<H1>XML support is unavailable</H1>Please install the XML::Twig module or disable traditional mode (drop the '-t') 
           in order to use this functionality.
           <BR>";
    } elsif($error==7) {
    print "<H1>Internal error, you found a bug!</H1>HTTPush encountered an internal error, the following details could help
to find where it came from:<BR>
<B><PRE>$data</PRE></B><BR>Please submit this information to our <A
HREF=\"http://sourceforge.net/tracker/?func=add&group_id=29026&atid=395271\">Bug Reporting System</A> to help us fix it in a future
release.<BR>Thanks for your support.";
    } else {
    print "<H1>There was an undefined error while trying to process your
           request.</H1><BR>";
    }

  print "<HR><ADDRESS>HTTPush/$VERSION at $HTTPUSH_OPTIONS{'listen_ip'} Port $HTTPUSH_OPTIONS{'listen_port'}</ADDRESS>";

  print "</BODY></HTML>";

  if($HTTPUSH_OPTIONS{'ssl'}) {
    Net::SSLeay::CTX_free ($CURRENT{'ctx'});
  }

  close Client;
}

return 1;
